@@ -30,6 +30,8 @@
 #ifndef _CPP_BITS_BASICIOS_H
 #define _CPP_BITS_BASICIOS_H 1
 
+#include <bits/sbuf_iter.h>
+
 namespace std {
 
   // 27.4.5  Template class basic_ios
@@ -45,7 +47,12 @@ namespace std {
 
       // Non-standard Types:
       typedef ctype<_CharT>           		__ctype_type;
-     
+      // From ostream
+      typedef ostreambuf_iterator<_CharT>		__ostreambuf_iter;
+      typedef num_put<_CharT, __ostreambuf_iter>        __numput_type;
+      typedef istreambuf_iterator<_CharT>		__istreambuf_iter;
+      typedef num_get<_CharT, __istreambuf_iter>        __numget_type;
+      
       // Data members:
     private:
       basic_ostream<_CharT, _Traits>* 	_M_tie;
@@ -58,57 +65,72 @@ namespace std {
 
       // Cached use_facet<ctype>, which is based on the current locale info.
       const __ctype_type*		_M_fctype_ios;      
+      // From ostream.
+      const __numput_type* 		_M_fnumput;
+      // From istream.
+      const __numget_type* 		_M_fnumget;
 
     public:
-      const __ctype_type*	
-      _M_get_fctype_ios()
+
+      inline const __ctype_type*	
+      _M_get_fctype_ios(void)
       { return _M_fctype_ios; }
 
+      inline const __numget_type* 
+      _M_get_fnumget(void)
+      { return _M_fnumget; }
+
+      inline const __numput_type* 
+      _M_get_fnumput(void)
+      { return _M_fnumput; }
+
       operator void*() const 
-      { return fail() ? 0 : const_cast<basic_ios*>(this); }
+      { return this->fail() ? 0 : const_cast<basic_ios*>(this); }
 
-      bool 
+      inline bool 
       operator!() const 
-      { return fail(); }
+      { return this->fail(); }
 
-      iostate 
+      inline iostate 
       rdstate() const 
-      {  return _M_streambuf_state; }
+      { return _M_streambuf_state; }
 
-      void 
+      inline void 
       clear(iostate __state = goodbit)
       { 
 	if (this->rdbuf())
 	  _M_streambuf_state = __state;
 	else
 	  _M_streambuf_state = __state | badbit;
+	if ((this->rdstate() & this->exceptions()))
+	  throw failure("basic_ios::clear(iostate) caused exception");
       }
 
-      void 
+      inline void 
       setstate(iostate __state) 
-      { this->clear(rdstate() | __state); }
+      { this->clear(this->rdstate() | __state); }
 
-      bool 
+      inline bool 
       good() const 
-      { return rdstate() == 0; }
+      { return this->rdstate() == 0; }
 
-      bool 
+      inline bool 
       eof() const 
-      { return (rdstate() & eofbit) != 0; }
+      { return (this->rdstate() & eofbit) != 0; }
 
-      bool 
+      inline bool 
       fail() const 
-      { return (rdstate() & (badbit | failbit)) != 0; }
+      { return (this->rdstate() & (badbit | failbit)) != 0; }
 
-      bool 
+      inline bool 
       bad() const 
-      { return (rdstate() & badbit) != 0; }
+      { return (this->rdstate() & badbit) != 0; }
 
-      iostate 
+      inline iostate 
       exceptions() const 
       { return _M_exception; }
 
-      void 
+      inline void 
       exceptions(iostate __except) 
       { 
 	_M_exception = __except; 
@@ -117,36 +139,46 @@ namespace std {
 
       // Constructor/destructor:
       explicit 
-      basic_ios(basic_streambuf<_CharT,_Traits>* __sb) : ios_base() 
+      basic_ios(basic_streambuf<_CharT, _Traits>* __sb) : ios_base() 
       { this->init(__sb); }
 
       virtual 
       ~basic_ios() { }
       
       // Members:
-      basic_ostream<_CharT, _Traits>*
+      inline basic_ostream<_CharT, _Traits>*
       tie() const      
       { return _M_tie; }
 
-      basic_ostream<_CharT, _Traits>*
-      tie(basic_ostream<_CharT,_Traits>* __tiestr);
+      inline basic_ostream<_CharT, _Traits>*
+      tie(basic_ostream<_CharT, _Traits>* __tiestr)
+      {
+	basic_ostream<_CharT, _Traits>* __old = _M_tie;
+	_M_tie = __tiestr;
+	return __old;
+      }
 
-      basic_streambuf<_CharT, _Traits>*
+      inline basic_streambuf<_CharT, _Traits>*
       rdbuf() const    
       { return _M_streambuf; }
 
       basic_streambuf<_CharT, _Traits>* 
-      rdbuf(basic_streambuf<_CharT,_Traits>* __sb);
+      rdbuf(basic_streambuf<_CharT, _Traits>* __sb);
 
       basic_ios&
       copyfmt(const basic_ios& __rhs);
 
-      char_type 
+      inline char_type 
       fill() const 
       { return _M_fill; }
 
-      char_type 
-      fill(char_type __ch);
+      inline char_type 
+      fill(char_type __ch)
+      {
+	char_type __old = _M_fill;
+	_M_fill = __ch;
+	return __old;
+      }
 
       // Locales:
       locale 
@@ -157,106 +189,23 @@ namespace std {
 
       char_type 
       widen(char __c) const;
-      
+     
     protected:
       // 27.4.5.1  basic_ios constructors
       basic_ios() : ios_base() 
       { }
 
       void 
-      init(basic_streambuf<_CharT,_Traits>* __sb);
+      init(basic_streambuf<_CharT, _Traits>* __sb);
     };
-
-  template<typename _CharT, typename _Traits>
-    basic_ostream<_CharT, _Traits>*
-    basic_ios<_CharT, _Traits>::tie(basic_ostream<_CharT, _Traits>* __tiestr)
-    {
-      basic_ostream<_CharT, _Traits>* __old = _M_tie;
-      _M_tie = __tiestr;
-      return __old;
-    }
-
-  template<typename _CharT, typename _Traits>
-    basic_streambuf<_CharT, _Traits>* 
-    basic_ios<_CharT, _Traits>::rdbuf(basic_streambuf<_CharT, _Traits>* __sb)
-    {
-      basic_streambuf<_CharT, _Traits>* __old = _M_streambuf;
-      _M_streambuf = __sb;
-      this->clear();
-      return __old;
-    }
-
-  // XXX This should be "export", and moved to bits/basic_ios.tcc.
-  template<typename _CharT, typename _Traits>
-    basic_ios<_CharT, _Traits>&
-    basic_ios<_CharT, _Traits>::copyfmt(const basic_ios& __rhs)
-    {
-      // Per 27.1.1.1, do not call imbue();  must not imbue on the sbuf.
-      // XXX but must trash all caches associated with imbue()
-      
-      // XXX shouldn't _M_copy_base take a const argument?
-      _M_copy_base(const_cast<basic_ios&> (__rhs));  
-      
-      tie(__rhs.tie());
-      fill(__rhs.fill());
-
-      // The next is required to be the last assignment.
-      exceptions(__rhs.exceptions());
-      
-      _M_call_callbacks(copyfmt_event);
-      return *this;
-    }
-
-  template<typename _CharT, typename _Traits>
-    _CharT 
-    basic_ios<_CharT, _Traits>::fill(char_type __ch)
-    {
-      char_type __old = _M_fill;
-      _M_fill = __ch;
-      return __old;
-    }
-
-  // Locales:
-  template<typename _CharT, typename _Traits>
-    locale
-    basic_ios<_CharT, _Traits>::imbue(const locale& __loc)
-    {
-      locale __old(getloc());
-      ios_base::imbue(__loc);
-      _M_fctype_ios =  &use_facet<__ctype_type>(__loc);
-      if (rdbuf() != 0)
-	rdbuf()->pubimbue(__loc);
-      _M_call_callbacks(imbue_event);
-      return __old;
-    }
-
-  template<typename _CharT, typename _Traits>
-    char
-    basic_ios<_CharT, _Traits>::narrow(char_type __c, char __dfault) const
-    { return _M_fctype_ios->narrow(__c, __dfault); }
-
-  template<typename _CharT, typename _Traits>
-    _CharT
-    basic_ios<_CharT, _Traits>::widen(char __c) const
-    { return _M_fctype_ios->widen(__c); }
-
-  template<typename _CharT, typename _Traits>
-    void
-    basic_ios<_CharT, _Traits>::init(basic_streambuf<_CharT, _Traits>* __sb)
-    {
-      // NB: This may be called more than once on the same object.
-      ios_base::_M_init();
-      _M_fctype_ios =  &use_facet<__ctype_type>(this->getloc());
-      _M_tie = 0;
-      _M_fill = this->widen(' ');
-      _M_exception = goodbit;
-      _M_streambuf = __sb;
-      iostate __state = __sb ? goodbit : badbit;
-      _M_streambuf_state = __state;
-    }
   
 } // namespace std
 
+#ifdef _GLIBCPP_NO_TEMPLATE_EXPORT
+# define export
+//#include <bits/basic_ios.tcc>
+#endif
+
 #endif /* _CPP_BITS_BASICIOS_H */
 
 
