@@ -49,34 +49,36 @@ extern "C" {
 namespace std {
 
   // 27.4.1  Types
-  typedef _IO_off_t  streamoff;
-  typedef _IO_ssize_t streamsize; // Signed integral type
-  typedef _IO_off_t  wstreamoff;
-  typedef _IO_ssize_t wstreamsize;
+  typedef _IO_off_t  	streamoff;
+  typedef _IO_ssize_t 	streamsize; // Signed integral type
+  typedef _IO_off_t  	wstreamoff;
+  typedef _IO_ssize_t 	wstreamsize;
 
   // 27.4.3  Template class fpos
   template<typename _StateT>
     class fpos
     {
+      // Types:
+      typedef _StateT __state_type;
+
     public:
-      _StateT 
+      __state_type
       state() const  { return _M_st; }
 
       void 
-      state(_StateT __st)  { _M_st = __st; }
+      state(__state_type __st)  { _M_st = __st; }
 
-      // NB: the standard defines only the implicit copy ctor and the
+      // NB: The standard defines only the implicit copy ctor and the
       // previous two members.  The rest is a "conforming extension".
-      fpos() { }
+      fpos(): _M_st(__state_type()), _M_pos(streamoff()) { }
 
-      fpos(streamoff __pos, _StateT __st)
+      fpos(streamoff __pos, __state_type __st)
       : _M_st(__st), _M_pos(__pos) { }
 
       fpos(streamoff __pos)
       : _M_st(), _M_pos(__pos) { }
-      // More are needed where streamoff is not an integer type.
 
-      operator streamoff&() { return _M_pos; }
+      operator streamoff() const { return _M_pos; }
 
       fpos& 
       operator+=(streamoff __off) { _M_pos += __off; return *this; }
@@ -85,25 +87,25 @@ namespace std {
       operator-=(streamoff __off) { _M_pos -= __off; return *this; }
 
       bool  
-      operator==(const fpos& __pos2) { return _M_pos == __pos2._M_pos; }
+      operator==(const fpos& __pos2) const { return _M_pos == __pos2._M_pos; }
 
       bool  
-      operator!=(const fpos& __pos2) { return _M_pos != __pos2._M_pos; }
+      operator!=(const fpos& __pos2) const { return _M_pos != __pos2._M_pos; }
       
       streamoff 
-      _M_position ()            { return _M_pos; }
+      _M_position() const { return _M_pos; }
 
       void
-      _M_position (streamoff __pos)  { _M_pos = __pos; }
+      _M_position(streamoff __pos)  { _M_pos = __pos; }
 
     private:
-      _StateT _M_st;
+      __state_type _M_st;
       streamoff _M_pos;
     };
 
   template<typename _State>
     inline fpos<_State> 
-    operator+(fpos<_State> const& __pos, streamoff __off)
+    operator+(const fpos<_State>& __pos, streamoff __off)
     { 
       fpos<_State> t(__pos); 
       return t += __off; 
@@ -111,7 +113,7 @@ namespace std {
 
   template<typename _State>
     inline fpos<_State>
-    operator-(fpos<_State> const& __pos, streamoff __off)
+    operator-(const fpos<_State>& __pos, streamoff __off)
     { 
       fpos<_State> t(__pos); 
       return t -= __off; 
@@ -119,10 +121,11 @@ namespace std {
 
   template<typename _State>
     inline streamoff 
-    operator-(fpos<_State> const& __pos1, fpos<_State> const& __pos2)
+    operator-(const fpos<_State>& __pos1, const fpos<_State>& __pos2)
     { return __pos1._M_position() - __pos2._M_position(); }
 
 }  // namespace std
 
 #endif /* _CPP_BITS_FPOS_H */
 
+
