@@ -77,39 +77,7 @@ namespace std
     { throw _Bad_use_facet(); }
 
   // 22.2.1  The ctype category
-  struct ctype_base
-  {
-    enum
-    {
-#ifdef _GLIBCPP_USE_CTYPE_ISBIT
-      space = _ISspace,
-      print = _ISprint,
-      cntrl = _IScntrl,
-      upper = _ISupper,
-      lower = _ISlower,
-      alpha = _ISalpha,
-      digit = _ISdigit,
-      punct = _ISpunct,
-      xdigit = _ISxdigit,
-      alnum = _ISalnum,
-      graph = _ISgraph
-#else
-      space = (1 << 5),	// Whitespace
-      print = (1 << 6),	// Printing
-      cntrl = (1 << 9),	// Control character
-      upper = (1 << 0),	// UPPERCASE
-      lower = (1 << 1),	// lowercase
-      alpha = (1 << 5),	// Alphabetic
-      digit = (1 << 2),	// Numeric
-      punct = (1 << 10),// Punctuation
-      xdigit = (1 << 4),// Hexadecimal numeric
-      alnum = (1 << 11),// Alphanumeric
-      graph = (1 << 7)	// Graphical
-#endif
-    };
-    typedef unsigned short mask;
-  };
-
+#include <bits/ctype_base.h>
 
   // 22.2.1.1  Template class ctype
   // _Ctype_nois is the common base for ctype<char>.  It lacks "do_is"
@@ -252,28 +220,25 @@ namespace std
       ~ctype() { }
 
       virtual bool 
-      do_is(mask __m, char_type __c) const
+      do_is(mask , char_type) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
 
       virtual const char_type*
-      do_is(const char_type* __lo, const char_type* __hi, 
-	    mask* __vec) const
+      do_is(const char_type*, const char_type*, mask*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
 
       virtual const char_type*
-      do_scan_is(mask __m, const char_type* __lo, 
-		 const char_type* __hi) const
+      do_scan_is(mask, const char_type*, const char_type*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
 
       virtual const char_type*
-      do_scan_not(mask __m, const char_type* __lo, 
-		  const char_type* __hi) const
+      do_scan_not(mask, const char_type*, const char_type*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
@@ -285,7 +250,7 @@ namespace std
       }
 
       virtual const char_type*
-      do_toupper(char_type* __low, const char_type* __high) const
+      do_toupper(char_type*, const char_type*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
@@ -297,7 +262,7 @@ namespace std
       }
 
       virtual const char_type*
-      do_tolower(char_type* __low, const char_type* __high) const
+      do_tolower(char_type*, const char_type*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
@@ -309,21 +274,19 @@ namespace std
       }
 
       virtual const char*
-      do_widen(const char* __low, const char* __high,
-	       char_type* __dest) const
+      do_widen(const char*, const char*, char_type*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
 
       virtual char 
-      do_narrow(char_type, char __dfault) const
+      do_narrow(char_type, char) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
 
       virtual const char_type*
-      do_narrow(const char_type* __low, const char_type* __high,
-		 char __dfault, char* __dest) const
+      do_narrow(const char_type*, const char_type*, char, char*) const
       {
 	// XXX Need definitions for these abstract mf's.
       }
@@ -343,13 +306,13 @@ namespace std
       // Data Members:
       const mask* 		_M_table;
       bool 			_M_del;
-      static const int* const& 	_S_toupper;
-      static const int* const& 	_S_tolower;
+      static __to_type const& 	_S_toupper;
+      static __to_type const& 	_S_tolower;
       static const mask* const& _S_table;
       
     public:
       static locale::id 	id;
-      static const size_t 	table_size = 1u + ~(unsigned char)0;
+      static const size_t      table_size = 1 + static_cast<unsigned char>(-1);
 
       explicit 
       ctype(const mask* __table = 0, bool __del = false, 
@@ -427,8 +390,8 @@ namespace std
     };
 
   template<>
-    inline const ctype<char>&
-    use_facet< const ctype<char> > (const locale& __loc);
+    const ctype<char>&
+    use_facet<const ctype<char> > (const locale& __loc);
 
 #ifdef _GLIBCPP_USE_WCHAR_T
   // ctype<wchar_t> specialization
@@ -443,8 +406,8 @@ namespace std
       static locale::id id;
 
     private:
-      static const int* const& _S_toupper;
-      static const int* const& _S_tolower;
+      static __to_type const& _S_toupper;
+      static __to_type const& _S_tolower;
       static const mask* const& _S_table;
       static const int _S_table_size = ctype<char>::table_size;
       
@@ -496,7 +459,7 @@ namespace std
     };
 
   template<>
-    inline const ctype<wchar_t>&
+    const ctype<wchar_t>&
     use_facet< const ctype<wchar_t> > (const locale& __loc);
 #endif //_GLIBCPP_USE_WCHAR_T
 
@@ -507,8 +470,8 @@ namespace std
       typedef _CharT char_type;
     public:
       explicit 
-      ctype_byname(const char*, size_t __refs = 0)
-      : ctype<_CharT>(__refs) { }
+      ctype_byname(const char*, size_t __refs = 0);
+
     protected:
       virtual 
       ~ctype_byname() { }
@@ -1089,7 +1052,7 @@ namespace std
       { return do_put(__s, __f, __fill, __v); }
 
       iter_type 
-      put (iter_type __s, ios_base& __f, char_type __fill, long __v) const
+      put(iter_type __s, ios_base& __f, char_type __fill, long __v) const
       { return do_put(__s, __f, __fill, __v); }
 
 #ifdef _GLIBCPP_USE_LONG_LONG 
@@ -1290,8 +1253,7 @@ namespace std
       typedef basic_string<_CharT> string_type;
 
       explicit 
-      numpunct_byname(const char*, size_t __refs = 0)
-      : numpunct<_CharT> (__refs) { }
+      numpunct_byname(const char*, size_t __refs = 0);
       
     protected:
       virtual 
@@ -1439,7 +1401,7 @@ namespace std
     collate_byname<char>::collate_byname(const char*, size_t __refs);
 #ifdef _GLIBCPP_USE_WCHAR_T
   template<>
-    collate_byname<wchar_t>::collate_byname (const char*, size_t __refs);
+    collate_byname<wchar_t>::collate_byname(const char*, size_t __refs);
 #endif
 
   class time_base
@@ -1675,6 +1637,8 @@ namespace std
   {
     enum part { none, space, symbol, sign, value };
     struct pattern { char field[4]; };
+
+    static const pattern __default_pattern;
   };
 
   template<typename _CharT>
@@ -1735,15 +1699,13 @@ namespace std
       virtual pattern      
       do_pos_format() const
       {
-	static const money_base::pattern __pat = {{symbol, sign, none, value}};
-	return __pat;
+	return money_base::__default_pattern;
       }
 
       virtual pattern      
       do_neg_format() const
       {
-	static const money_base::pattern __pat = {{symbol, sign, none, value}};
-	return __pat;
+	return money_base::__default_pattern;
       }
 
     };
@@ -1774,8 +1736,7 @@ namespace std
       static const bool intl = _Intl;
 
       explicit 
-      moneypunct_byname(const char*, size_t __refs = 0)
-      : moneypunct<_CharT,_Intl> (__refs) { }
+      moneypunct_byname(const char*, size_t __refs = 0);
 
     protected:
       virtual 
@@ -1864,8 +1825,7 @@ namespace std
       typedef basic_string<_CharT> string_type;
 
       explicit 
-      messages_byname(const char*, size_t __refs = 0) 
-      : messages<_CharT>(__refs) { }
+      messages_byname(const char*, size_t __refs = 0);
 
     protected:
       virtual 
