@@ -108,7 +108,7 @@ namespace std {
     basic_istream<_CharT,_Traits>& 
     operator>>(basic_istream<_CharT,_Traits>& __is, _Setbase __f)
     {
-      is.setf(__f._M_base ==  8 ? ios_base::oct : 
+      __is.setf(__f._M_base ==  8 ? ios_base::oct : 
 	      __f._M_base == 10 ? ios_base::dec : 
 	      __f._M_base == 16 ? ios_base::hex : 
 	      ios_base::fmtflags(0), ios_base::basefield);
