@@ -44,13 +44,15 @@ namespace std {
   extern ostream cout;
   extern ostream cerr;
   extern ostream clog;
-#ifdef _G_USE_WCHAR_T
+#ifdef _GLIBCPP_USE_WCHAR_T
   extern wistream wcin;
   extern wostream wcout;
   extern wostream wcerr;
   extern wostream wclog;
 #endif
 
+  // For construction of filebuffers for cout, cin, cerr, clog et. al.
+  static ios_base::Init __ioinit;
 } // namespace std
 
 #endif	/* _CPP_IOSTREAM */
