@@ -40,8 +40,6 @@
 #include <bits/std_functional.h>
 #include <bits/std_algorithm.h>
 
-#include <alloca.h>  // XXX non-standard.  Used also in bits/gslice.h
-
 namespace std {
 
     template<class _Clos, typename _Tp> class _Expr;
@@ -98,7 +96,7 @@ namespace std {
       valarray();
       explicit valarray(size_t);
       valarray(const _Tp&, size_t);
-      valarray(const _Tp*, size_t);
+      valarray(const _Tp* __restrict__, size_t);
       valarray(const valarray&);
       valarray(const slice_array<_Tp>&);
       valarray(const gslice_array<_Tp>&);
@@ -189,8 +187,8 @@ namespace std {
       _Tp    min() const;	
       _Tp    max() const;	
 
-          // FIXME: Extension
-      _Tp    product () const;
+//           // FIXME: Extension
+//       _Tp    product () const;
 
       valarray<_Tp> shift (int) const;
       valarray<_Tp> cshift(int) const;
@@ -200,7 +198,7 @@ namespace std {
 
   private:
       size_t _M_size;
-      _Tp* _M_data;
+      _Tp* __restrict__ _M_data;
 
       friend class _Array<_Tp>;
   };
@@ -241,7 +239,7 @@ namespace std {
   { return _M_data[__i]; }
 
   template<typename _Tp>
-  inline _Tp&
+  _Tp&
   valarray<_Tp>::operator[] (size_t __i)
   { return _M_data[__i]; }
 
@@ -269,7 +267,7 @@ namespace std {
   { __valarray_fill (_M_data, _M_size, __t); }
 
   template<typename _Tp>
-  inline valarray<_Tp>::valarray (const _Tp* __pT, size_t __n)
+  inline valarray<_Tp>::valarray (const _Tp* __restrict__ __pT, size_t __n)
       : _M_size (__n), _M_data (new _Tp[__n])
   { __valarray_copy (__pT, __n, _M_data); }
 
@@ -286,8 +284,8 @@ namespace std {
 
   template<typename _Tp>
   inline valarray<_Tp>::valarray (const gslice_array<_Tp>& __ga)
-      : _M_size (__ga._M_gs._M_index_size), _M_data (new _Tp[_M_size])
-  { __valarray_copy (__ga._M_array, __ga._M_gs._M_index, 
+      : _M_size (__ga._M_index.size()), _M_data (new _Tp[_M_size])
+  { __valarray_copy (__ga._M_array, _Array<size_t>(__ga._M_index), 
             _Array<_Tp>(_M_data), _M_size); }
 
   template<typename _Tp>
@@ -339,7 +337,7 @@ namespace std {
   inline valarray<_Tp>&
   valarray<_Tp>::operator= (const gslice_array<_Tp>& __ga)
   {
-      __valarray_copy (__ga._M_array, __ga._M_gs._M_index,
+      __valarray_copy (__ga._M_array, _Array<size_t>(__ga._M_index),
               _Array<_Tp>(_M_data), _M_size);
       return *this;
   }
@@ -390,14 +388,16 @@ namespace std {
   valarray<_Tp>::operator[] (const gslice& __gs) const
   {
       typedef _GClos<_ValArray,_Tp> _Closure;
-      return _Expr<_Closure, _Tp> (_Closure (_Array<_Tp>(_M_data), __gs));
+      return _Expr<_Closure, _Tp>
+          (_Closure (_Array<_Tp>(_M_data), __gs._M_index->_M_index));
   }
 
   template<typename _Tp>
   inline gslice_array<_Tp>
   valarray<_Tp>::operator[] (const gslice& __gs)
   {
-      return gslice_array<_Tp> (_Array<_Tp>(_M_data), __gs);
+      return gslice_array<_Tp>
+          (_Array<_Tp>(_M_data), __gs._M_index->_M_index);
   }
 
   template<typename _Tp>
@@ -446,21 +446,22 @@ namespace std {
   inline _Tp
   valarray<_Tp>::sum () const
   {
-      return accumulate (_M_data, _M_data + _M_size, _Tp ());
+      return __valarray_sum(_M_data, _M_data + _M_size);
   }
 
-  template<typename _Tp>
-  inline _Tp
-  valarray<_Tp>::product () const
-  {
-      return accumulate (_M_data, _M_data+_M_size, _Tp(1), multiplies<_Tp> ());
-  }
+//   template<typename _Tp>
+//   inline _Tp
+//   valarray<_Tp>::product () const
+//   {
+//       return __valarray_product(_M_data, _M_data + _M_size);
+//   }
 
   template <class _Tp>
   inline valarray<_Tp>
   valarray<_Tp>::shift (int __n) const
   {
-      _Tp* const __a = static_cast<_Tp*> (alloca (sizeof(_Tp) * _M_size));
+      _Tp* const __a = static_cast<_Tp*>
+          (__builtin_alloca (sizeof(_Tp) * _M_size));
       if (! __n)                          // __n == 0: no shift
         __valarray_copy (_M_data, _M_size, __a);
       else if (__n > 0) {                  // __n > 0: shift left
@@ -482,7 +483,8 @@ namespace std {
   inline valarray<_Tp>
   valarray<_Tp>::cshift (int __n) const
   {
-      _Tp* const __a = static_cast<_Tp*> (alloca (sizeof(_Tp) * _M_size));
+      _Tp* const __a = static_cast<_Tp*>
+          (__builtin_alloca (sizeof(_Tp) * _M_size));
       if (! __n)                          // __n == 0: no cshift
         __valarray_copy(_M_data, _M_size, __a);
       else if (__n > 0) {                 // __n > 0: cshift left
