@@ -98,7 +98,9 @@ namespace std {
 		     int __prot = 0664)
   {
     __basic_file* __retval = NULL;
+#ifdef O_BINARY
     bool __testb = __mode & ios_base::binary;
+#endif
     bool __testi = __mode & ios_base::in;
     bool __testo = __mode & ios_base::out;
     bool __testt = __mode & ios_base::trunc;
@@ -108,12 +110,12 @@ namespace std {
     
     if (!__testi && __testo && !__testt && !__testa)
       {
-	__p_mode = O_WRONLY;
+	__p_mode = O_WRONLY | O_TRUNC | O_CREAT;
 	__rw_mode = _IO_NO_READS;
       }
     if (!__testi && __testo && !__testt && __testa)
       {
-	__p_mode = O_WRONLY | O_APPEND;
+	__p_mode = O_WRONLY | O_APPEND | O_CREAT;
 	__rw_mode = _IO_NO_READS | _IO_IS_APPENDING;
       }
     if (!__testi && __testo && __testt && !__testa)
@@ -222,7 +224,7 @@ namespace std {
   { return _IO_file_write(this, __s, __n); }
 
   __c_streampos 
-  __basic_file::sys_seek(streamoff __off, ios_base::seekdir __way)
+  __basic_file::sys_seek(__c_streampos __off, ios_base::seekdir __way)
   { return _IO_file_seek(this, __off, __way); }
   
   int 
@@ -235,7 +237,7 @@ namespace std {
   __basic_file::showmanyc() { return -1; }
 
   void 
-  __basic_file::imbue(void* __v) { }
+  __basic_file::imbue(void* /*__v*/) { }
 
 }  // namespace std
 
