@@ -38,10 +38,9 @@
 #include <bits/std_cstddef.h>     // for size_t
 #include <bits/std_string.h>
 #include <bits/std_stdexcept.h>
-
 #include <bits/std_istream.h>
 #include <bits/std_ostream.h>
-
+#include <bits/std_algorithm.h>
 
 #define __BITS_PER_WORDT(__wt) (CHAR_BIT*sizeof(__wt))
 #define __BITSET_WORDS(__n,__wt) \
@@ -557,7 +556,6 @@ private:
   // name resolution rules.
 
 #ifdef __STL_HAS_NAMESPACES
-
   using _Base::_S_whichword;
   using _Base::_S_whichbyte;
   using _Base::_S_whichbit;
@@ -578,7 +576,6 @@ private:
   using _Base::_M_do_to_ulong;
   using _Base::_M_do_find_first;
   using _Base::_M_do_find_next;
-
 #endif /* __STL_HAS_NAMESPACES */
 
 private:
@@ -648,19 +645,7 @@ public:
   bitset(unsigned long __val) :
     _Base_bitset<__BITSET_WORDS(_Nb,_WordT), _WordT>(__val) {}
 
-  template<class _CharT, class _Traits, class _Alloc>
-  explicit bitset(const basic_string<_CharT,_Traits,_Alloc>& __s,
-                  size_t __pos = 0, 
-		  size_t __n = basic_string<_CharT,_Traits,_Alloc>::npos) 
-    : _Base()
-  {
-    if (__pos > __s.size())
-      __STL_THROW(out_of_range("bitset"));
-    _M_copy_from_string(__s, __pos, __n);
-  }
-
 #ifdef __STL_MEMBER_TEMPLATES
-
   template<class _CharT, class _Traits, class _Alloc>
   explicit bitset(const basic_string<_CharT,_Traits,_Alloc>& __s,
                   size_t __pos = 0)
@@ -682,9 +667,7 @@ public:
 	__STL_THROW(out_of_range("bitset"));
       _M_copy_from_string(__s, __pos, __n);
     }
-
-#else /* __STL_MEMBER_TEMPLATES */
-
+#else 
   explicit bitset(const basic_string<char>& __s,
 		  size_t __pos = 0,
 		  size_t __n = basic_string<char>::npos) 
