@@ -183,10 +183,38 @@ inline _OutputIter __copy_aux(_InputIter __first, _InputIter __last,
   return __copy_aux2(__first, __last, __result, _Trivial());
 }
 
+template<typename _InputIter, typename _OutputIter>
+inline _OutputIter __copy_ni2(_InputIter __first, _InputIter __last,
+                               _OutputIter __result, __true_type) {
+  return _OutputIter(__copy_aux(__first, __last, __result.base(),
+                                __VALUE_TYPE(__first)));
+}
+
+template<typename _InputIter, typename _OutputIter>
+inline _OutputIter __copy_ni2(_InputIter __first, _InputIter __last,
+			      _OutputIter __result, __false_type) {
+  return __copy_aux(__first, __last, __result, __VALUE_TYPE(__first));
+}
+
+template<typename _InputIter, typename _OutputIter>
+inline _OutputIter __copy_ni1(_InputIter __first, _InputIter __last,
+                               _OutputIter __result, __true_type) {
+  typedef typename _Is_normal_iterator<_OutputIter>::_Normal __Normal;
+  return __copy_ni2(__first.base(), __last.base(), __result, __Normal());
+}
+
+template<typename _InputIter, typename _OutputIter>
+inline _OutputIter __copy_ni1(_InputIter __first, _InputIter __last,
+                               _OutputIter __result, __false_type) {
+  typedef typename _Is_normal_iterator<_OutputIter>::_Normal __Normal;
+  return __copy_ni2(__first, __last, __result, __Normal());
+}
+
 template <class _InputIter, class _OutputIter>
 inline _OutputIter copy(_InputIter __first, _InputIter __last,
                         _OutputIter __result) {
-  return __copy_aux(__first, __last, __result, __VALUE_TYPE(__first));
+   typedef typename _Is_normal_iterator<_InputIter>::_Normal __Normal;
+   return __copy_ni1(__first, __last, __result, __Normal());
 }
 
 // Hack for compilers that don't have partial ordering of function templates
