@@ -14,6 +14,7 @@
 #ifndef __SGI_STL_STRING_FWD_H
 #define __SGI_STL_STRING_FWD_H
 
+#include <bits/stl_config.h>
 #include <bits/std_cstddef.h>
 #include <bits/stl_alloc.h>
 
@@ -26,9 +27,9 @@ template <class _CharT,
 class basic_string;
 
 typedef basic_string<char>    string;
+#ifdef _GLIBCPP_USE_WCHAR_T
 typedef basic_string<wchar_t> wstring;
-
-const char* __get_c_string(const string&);
+#endif
 
 __STL_END_NAMESPACE
 
