@@ -35,9 +35,16 @@ void test02()
   sin(static_cast<float>(0));
 }
 
+// as did this.
+void test03()
+{
+  double powtest = pow(2., 0);
+}
+
 int main()
 {
     test01();
     test02();
+    test03();
     return 0;
 }
