@@ -231,8 +231,7 @@ bool test04() {
   c2 = strb_01.sputbackc('z');//"mykonos. . .zor what?"
   c3 = strb_01.sgetc();
   test &= c1 != c2;
-  test &= c3 != c2;
-  test &= c1 == c3;
+  test &= c3 == c2;
   test &= strb_01.str() == std::string("mykonos. . .zor what?");
   test &= str_tmp.size() == strb_01.str().size();
   //test for _in_cur == _in_beg
@@ -338,7 +337,8 @@ bool test04() {
   strb_03.str(str_03); //out ("")
   //IN|OUT
   //beg
-  pt_1 = strb_01.pubseekoff(2, std::ios_base::beg, std::ios_base::in|std::ios_base::out);
+  pt_1 = strb_01.pubseekoff(2, std::ios_base::beg, 
+			    std::ios_base::in|std::ios_base::out);
   off_1 = pt_1._M_position();
   test &= off_1 > 0;
   c1 = strb_01.snextc(); //current in pointer +1
